Attribute VB_Name = "MathGammaF"
Option Explicit

Private Const ITMAX As Integer = 100
Private Const EPS As Single = 0.0000003!

Public Function GammLn(ByVal XX As Single) As Single
    Const C_0 As Double = 76.1800917294715
    Const C_1 As Double = -86.5053203294168
    Const C_2 As Double = 24.0140982408309
    Const C_3 As Double = -1.23173957245015
    Const C_4 As Double = 1.20865097386618E-03
    Const C_5 As Double = -5.395239384953E-06
    
    Dim x As Double, Y As Double, Tmp As Double, Ser As Double
    Dim I As Integer, J As Integer
    
    If XX <= 0# Then Err.Raise 5
    
    x = XX
    Y = XX
    Tmp = x + 5.5
    Tmp = Tmp - (x + 0.5) * Log(Tmp)
    Ser = 1.00000000019001
    
    Y = Y + 1#
    Ser = Ser + C_0 / Y
    Y = Y + 1#
    Ser = Ser + C_1 / Y
    Y = Y + 1#
    Ser = Ser + C_2 / Y
    Y = Y + 1#
    Ser = Ser + C_3 / Y
    Y = Y + 1#
    Ser = Ser + C_4 / Y
    Y = Y + 1#
    Ser = Ser + C_5 / Y
    
    GammLn = -Tmp + Log(2.506628274631 * Ser / x)
    
End Function

Public Function GammP(ByVal A As Single, ByVal x As Single) As Single
    Dim GamSer As Single, GamMcf As Single, Gln As Single
    
    If x < 0! Or A <= 0! Then Err.Raise 5
    
    If x < A + 1! Then
        GSer GamSer, A, x, Gln
        GammP = GamSer
    Else
        Gcf GamMcf, A, x, Gln
        GammP = 1! - GamMcf
    End If
    
End Function

Public Function GammQ(ByVal A As Single, ByVal x As Single) As Single
    Dim GamSer As Single, GamMcf As Single, Gln As Single
    
    If x < 0! Or A <= 0! Then Err.Raise 5
    
    If x < A + 1! Then
        GSer GamSer, A, x, Gln
        GammQ = 1! - GamSer
    Else
        Gcf GamMcf, A, x, Gln
        GammQ = GamMcf
    End If
    
End Function

Private Sub GSer(GamSer As Single, _
                 ByVal A As Single, _
                 ByVal x As Single, _
                 Gln As Single)
    Dim N As Integer
    Dim Sum As Single, Del As Single, Ap As Single
    
    Gln = GammLn(A)
    
    If x < 0! Then
        Err.Raise 5
    ElseIf x = 0! Then
        GamSer = 0!
    Else
        Ap = A
        Del = 1! / A
        Sum = Del
        
        For N = 1 To ITMAX
            Ap = Ap + 1!
            Del = Del * x / Ap
            Sum = Sum + Del
            If Abs(Del) < Abs(Sum) * EPS Then
                GamSer = Sum * Exp(-x + A * Log(x) - Gln)
                Exit Sub
            End If
        Next N
        
        Err.Raise 5
    End If
End Sub

Private Sub Gcf(GamMcf As Single, _
                ByVal A As Single, _
                ByVal x As Single, _
                Gln As Single)
    Dim N As Integer
    Dim Gold As Single, G As Single, Fac As Single, B1 As Single
    Dim B0 As Single, Anf As Single, Ana As Single, An As Single
    Dim A1 As Single, A0 As Single
    
    Fac = 1!
    B1 = 1!
    A0 = 1!
    
    Gln = GammLn(A)
    A1 = x
    
    For N = 1 To ITMAX
        An = CSng(N)
        Ana = An - A
        A0 = (A1 + A0 * Ana) * Fac
        B0 = (B1 + B0 * Ana) * Fac
        Anf = An * Fac
        A1 = x * A0 + Anf * A1
        B1 = x * B0 + Anf * B1
        If A1 > 0! Then
            Fac = 1! / A1
            G = B1 * Fac
            If Abs((G - Gold) / G) < EPS Then
                GamMcf = Exp(-x + A * Log(x) - Gln) * G
                Exit Sub
            End If
            Gold = G
        End If
    Next N
    
    Err.Raise 5
    
End Sub

